@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: GraphicsUtil.java 447277 2006-09-18 06:19:34Z jeremias $ */
+/* $Id: GraphicsUtil.java 499611 2007-01-24 23:17:50Z cam $ */
 
 package org.apache.xmlgraphics.image;
 
@@ -38,6 +38,7 @@ import java.awt.image.SampleModel;
 import java.awt.image.SinglePixelPackedSampleModel;
 import java.awt.image.WritableRaster;
 
+import org.apache.xmlgraphics.image.rendered.Any2LsRGBRed;
 import org.apache.xmlgraphics.image.rendered.Any2sRGBRed;
 import org.apache.xmlgraphics.image.rendered.BufferedImageCachableRed;
 import org.apache.xmlgraphics.image.rendered.CachableRed;
@@ -49,7 +50,7 @@ import org.apache.xmlgraphics.image.rendered.RenderedImageCachableRed;
  * implementations.
  *
  * @author <a href="mailto:Thomas.DeWeeese@Kodak.com">Thomas DeWeese</a>
- * @version $Id: GraphicsUtil.java 447277 2006-09-18 06:19:34Z jeremias $
+ * @version $Id: GraphicsUtil.java 499611 2007-01-24 23:17:50Z cam $
  */
 public class GraphicsUtil {
 
@@ -57,7 +58,7 @@ public class GraphicsUtil {
 
     /**
      * Standard prebuilt Linear_sRGB color model with no alpha */
-    public final static ColorModel Linear_sRGB =
+    public static final ColorModel Linear_sRGB =
         new DirectColorModel(ColorSpace.getInstance
                              (ColorSpace.CS_LINEAR_RGB), 24,
                              0x00FF0000, 0x0000FF00,
@@ -66,7 +67,7 @@ public class GraphicsUtil {
     /**
      * Standard prebuilt Linear_sRGB color model with premultiplied alpha.
      */
-    public final static ColorModel Linear_sRGB_Pre =
+    public static final ColorModel Linear_sRGB_Pre =
         new DirectColorModel(ColorSpace.getInstance
                              (ColorSpace.CS_LINEAR_RGB), 32,
                              0x00FF0000, 0x0000FF00,
@@ -75,7 +76,7 @@ public class GraphicsUtil {
     /**
      * Standard prebuilt Linear_sRGB color model with unpremultiplied alpha.
      */
-    public final static ColorModel Linear_sRGB_Unpre =
+    public static final ColorModel Linear_sRGB_Unpre =
         new DirectColorModel(ColorSpace.getInstance
                              (ColorSpace.CS_LINEAR_RGB), 32,
                              0x00FF0000, 0x0000FF00,
@@ -85,7 +86,7 @@ public class GraphicsUtil {
     /**
      * Standard prebuilt sRGB color model with no alpha.
      */
-    public final static ColorModel sRGB =
+    public static final ColorModel sRGB =
         new DirectColorModel(ColorSpace.getInstance
                              (ColorSpace.CS_sRGB), 24,
                              0x00FF0000, 0x0000FF00,
@@ -94,7 +95,7 @@ public class GraphicsUtil {
     /**
      * Standard prebuilt sRGB color model with premultiplied alpha.
      */
-    public final static ColorModel sRGB_Pre =
+    public static final ColorModel sRGB_Pre =
         new DirectColorModel(ColorSpace.getInstance
                              (ColorSpace.CS_sRGB), 32,
                              0x00FF0000, 0x0000FF00,
@@ -103,7 +104,7 @@ public class GraphicsUtil {
     /**
      * Standard prebuilt sRGB color model with unpremultiplied alpha.
      */
-    public final static ColorModel sRGB_Unpre =
+    public static final ColorModel sRGB_Unpre =
         new DirectColorModel(ColorSpace.getInstance
                              (ColorSpace.CS_sRGB), 32,
                              0x00FF0000, 0x0000FF00,
@@ -119,9 +120,7 @@ public class GraphicsUtil {
      *                <tt>premult</tt>
      */
     public static ColorModel makeLinear_sRGBCM(boolean premult) {
-        if (premult)
-            return Linear_sRGB_Pre;
-        return Linear_sRGB_Unpre;
+         return premult ? Linear_sRGB_Pre : Linear_sRGB_Unpre;
     }
 
     /**
@@ -139,6 +138,26 @@ public class GraphicsUtil {
         return new BufferedImage(cm, wr, premult, null);
     }
 
+    /**
+     * This method will return a CacheableRed that has it's data in
+     * the linear sRGB colorspace. If <tt>src</tt> is already in
+     * linear sRGB then this method does nothing and returns <tt>src</tt>.
+     * Otherwise it creates a transform that will convert
+     * <tt>src</tt>'s output to linear sRGB and returns that CacheableRed.
+     *
+     * @param src The image to convert to linear sRGB.
+     * @return    An equivilant image to <tt>src</tt> who's data is in
+     *            linear sRGB.
+     */
+    public static CachableRed convertToLsRGB(CachableRed src) {
+        ColorModel cm = src.getColorModel();
+        ColorSpace cs = cm.getColorSpace();
+        if (cs == ColorSpace.getInstance(ColorSpace.CS_LINEAR_RGB))
+            return src;
+
+        return new Any2LsRGBRed(src);
+    }
+
     /**
      * This method will return a CacheableRed that has it's data in
      * the sRGB colorspace. If <tt>src</tt> is already in
@@ -485,7 +504,7 @@ public class GraphicsUtil {
 
         // System.out.println("CoerceData: " + cm.isAlphaPremultiplied() +
         //                    " Out: " + newAlphaPreMult);
-        if (cm.hasAlpha()== false)
+        if (!cm.hasAlpha())
             // Nothing to do no alpha channel
             return cm;
 
@@ -532,7 +551,7 @@ public class GraphicsUtil {
                 }
         }
     }
-    
+
     public static void divideAlpha(WritableRaster wr) {
         if (is_BYTE_COMP_Data(wr.getSampleModel()))
             divide_BYTE_COMP_Data(wr);
@@ -543,7 +562,7 @@ public class GraphicsUtil {
             float ialpha;
             int    bands = wr.getNumBands();
             int [] pixel = null;
-        
+
             x0 = wr.getMinX();
             x1 = x0+wr.getWidth();
             y0 = wr.getMinY();
@@ -606,8 +625,8 @@ public class GraphicsUtil {
         //                    " is: " + dst.isAlphaPremultiplied());
 
         if (srcAlpha == dstAlpha)
-            if ((srcAlpha == false) ||
-                (src.isAlphaPremultiplied() == dst.isAlphaPremultiplied())) {
+            if (!srcAlpha ||
+                    src.isAlphaPremultiplied() == dst.isAlphaPremultiplied()) {
                 // They match one another so just copy everything...
                 copyData(src.getRaster(), dst.getRaster());
                 return;
@@ -858,20 +877,19 @@ public class GraphicsUtil {
             = (db.getOffset() +
                sppsm.getOffset(wr.getMinX()-wr.getSampleModelTranslateX(),
                                wr.getMinY()-wr.getSampleModelTranslateY()));
-        int pixel, a, aFP;
+
         // Access the pixel data array
-        final int pixels[] = db.getBankData()[0];
+        final int[] pixels = db.getBankData()[0];
         for (int y=0; y<wr.getHeight(); y++) {
             int sp = base + y*scanStride;
             final int end = sp + width;
             while (sp < end) {
-                pixel = pixels[sp];
-                a = pixel>>>24;
+                int pixel = pixels[sp];
+                int a = pixel>>>24;
                 if (a<=0) {
                     pixels[sp] = 0x00FFFFFF;
-                }
-                else if (a<255) {
-                    aFP = (0x00FF0000/a);
+                } else if (a<255) {
+                    int aFP = (0x00FF0000/a);
                     pixels[sp] =
                         ((a << 24) |
                          (((((pixel&0xFF0000)>>16)*aFP)&0xFF0000)    ) |
@@ -898,14 +916,14 @@ public class GraphicsUtil {
                sppsm.getOffset(wr.getMinX()-wr.getSampleModelTranslateX(),
                                wr.getMinY()-wr.getSampleModelTranslateY()));
         // Access the pixel data array
-        final int pixels[] = db.getBankData()[0];
+        final int[] pixels = db.getBankData()[0];
         for (int y=0; y<wr.getHeight(); y++) {
             int sp = base + y*scanStride;
             final int end = sp + width;
             while (sp < end) {
                 int pixel = pixels[sp];
                 int a = pixel>>>24;
-                if ((a>=0) && (a<255)) {
+                if ((a>=0) && (a<255)) {   // this does NOT include a == 255 (0xff) !
                     pixels[sp] = ((a << 24) |
                                   ((((pixel&0xFF0000)*a)>>8)&0xFF0000) |
                                   ((((pixel&0x00FF00)*a)>>8)&0x00FF00) |
@@ -935,25 +953,23 @@ public class GraphicsUtil {
                csm.getOffset(wr.getMinX()-wr.getSampleModelTranslateX(),
                              wr.getMinY()-wr.getSampleModelTranslateY()));
 
-
-        int a=0;
         int aOff = bandOff[bandOff.length-1];
         int bands = bandOff.length-1;
-        int b, i;
+
         // Access the pixel data array
-        final byte pixels[] = db.getBankData()[0];
+        final byte[] pixels = db.getBankData()[0];
         for (int y=0; y<wr.getHeight(); y++) {
             int sp = base + y*scanStride;
             final int end = sp + width*pixStride;
             while (sp < end) {
-              a = pixels[sp+aOff]&0xFF;
+              int a = pixels[sp+aOff]&0xFF;
               if (a==0) {
-                for (b=0; b<bands; b++)
+                for (int b = 0; b < bands; b++)
                   pixels[sp+bandOff[b]] = (byte)0xFF;
-              } else if (a<255) {
+              } else if (a<255) {         // this does NOT include a == 255 (0xff) !
                 int aFP = (0x00FF0000/a);
-                for (b=0; b<bands; b++) {
-                  i = sp+bandOff[b];
+                for (int b = 0; b < bands; b++) {
+                  int i = sp+bandOff[b];
                   pixels[i] = (byte)(((pixels[i]&0xFF)*aFP)>>>16);
                 }
               }
@@ -981,21 +997,19 @@ public class GraphicsUtil {
                              wr.getMinY()-wr.getSampleModelTranslateY()));
 
 
-        int a=0;
         int aOff = bandOff[bandOff.length-1];
         int bands = bandOff.length-1;
-        int b, i;
 
         // Access the pixel data array
-        final byte pixels[] = db.getBankData()[0];
+        final byte[] pixels = db.getBankData()[0];
         for (int y=0; y<wr.getHeight(); y++) {
             int sp = base + y*scanStride;
             final int end = sp + width*pixStride;
             while (sp < end) {
-              a = pixels[sp+aOff]&0xFF;
+              int a = pixels[sp+aOff]&0xFF;
               if (a!=0xFF)
-                for (b=0; b<bands; b++) {
-                  i = sp+bandOff[b];
+                for (int b = 0; b < bands; b++) {
+                  int i = sp+bandOff[b];
                   pixels[i] = (byte)(((pixels[i]&0xFF)*a)>>8);
                 }
               sp+=pixStride;
@@ -1013,7 +1027,7 @@ public class GraphicsUtil {
                 org.ImageDisplay.printImage
                     (label, subBI,
                      new Rectangle(75-iR.x, 90-iR.y, 32, 32));
-                
+
             }
 
 
@@ -1045,7 +1059,7 @@ public class GraphicsUtil {
                     System.out.println("BICM class: " + ri.getColorModel().getClass());
                     System.out.println("BICS: " + ri.getColorModel().getColorSpace());
                     System.out.println
-                        ("sRGB CS: " + 
+                        ("sRGB CS: " +
                          ColorSpace.getInstance(ColorSpace.CS_sRGB));
                     System.out.println("G2D info");
                     System.out.println("\tComposite: " + g2d.getComposite());
