@@ -15,11 +15,12 @@
  * limitations under the License.
  */
 
-/* $Id: PSGenerator.java 426584 2006-07-28 16:01:47Z jeremias $ */
+/* $Id: PSGenerator.java 549118 2007-06-20 14:27:03Z jeremias $ */
  
 package org.apache.xmlgraphics.ps;
 
 import java.awt.Color;
+import java.awt.color.ColorSpace;
 import java.awt.geom.AffineTransform;
 import java.io.OutputStream;
 import java.io.IOException;
@@ -27,31 +28,37 @@ import java.text.DateFormat;
 import java.text.DecimalFormat;
 import java.text.DecimalFormatSymbols;
 import java.util.Date;
-import java.util.Iterator;
 import java.util.Locale;
-import java.util.Set;
 import java.util.Stack;
 
 import javax.xml.transform.Source;
 
+import org.apache.xmlgraphics.ps.dsc.ResourceTracker;
+
 /**
  * This class is used to output PostScript code to an OutputStream.
  *
- * @version $Id: PSGenerator.java 426584 2006-07-28 16:01:47Z jeremias $
+ * @version $Id: PSGenerator.java 549118 2007-06-20 14:27:03Z jeremias $
  */
 public class PSGenerator {
 
+    /**
+     * Default postscript language level
+     */
+    public static final int DEFAULT_LANGUAGE_LEVEL = 3;
+    
     /** 
      * Indicator for the PostScript interpreter that the value is provided 
      * later in the document (mostly in the %%Trailer section).
+     * @deprecated Please use DSCConstants.ATEND. This constant was in the wrong place.
      */
-    public static final AtendIndicator ATEND = new AtendIndicator() {
-    };
+    public static final Object ATEND = DSCConstants.ATEND;
 
     /** Line feed used by PostScript */
     public static final char LF = '\n';
     
     private OutputStream out;
+    private int psLevel = DEFAULT_LANGUAGE_LEVEL;
     private boolean commentsEnabled = true;
     
     private Stack graphicsStateStack = new Stack();
@@ -62,7 +69,10 @@ public class PSGenerator {
 
     private StringBuffer tempBuffer = new StringBuffer(256);
 
-    /** @see java.io.FilterOutputStream **/
+    /**
+     * Creates a new instance.
+     * @param out the OutputStream to write the generated PostScript code to
+     */
     public PSGenerator(OutputStream out) {
         this.out = out;
         this.currentState = new PSState();
@@ -79,11 +89,18 @@ public class PSGenerator {
 
     /**
      * Returns the selected PostScript level. 
-     * (Hardcoded to level 2 for the moment.)
      * @return the PostScript level
      */
     public int getPSLevel() {
-        return 2; 
+        return this.psLevel; 
+    }
+    
+    /**
+     * Sets the PostScript level that is used to generate the current document.
+     * @param level the PostScript level (currently 1, 2 and 3 are known)
+     */
+    public void setPSLevel(int level) {
+        this.psLevel = level;
     }
     
     /**
@@ -185,7 +202,6 @@ public class PSGenerator {
         out.flush();
     }
 
-
     /**
      * Escapes a character conforming to the rules established in the PostScript
      * Language Reference (Search for "Literal Text Strings").
@@ -193,38 +209,45 @@ public class PSGenerator {
      * @param target target StringBuffer to write the escaped character to
      */
     public static final void escapeChar(char c, StringBuffer target) {
-        if (c > 127) {
-            target.append("\\");
-            target.append(Integer.toOctalString(c));
-        } else {
-            switch (c) {
-                case '\n':
-                    target.append("\\n");
-                    break;
-                case '\r':
-                    target.append("\\r");
-                    break;
-                case '\t':
-                    target.append("\\t");
-                    break;
-                case '\b':
-                    target.append("\\b");
-                    break;
-                case '\f':
-                    target.append("\\f");
-                    break;
-                case '\\':
-                    target.append("\\\\");
-                    break;
-                case '(':
-                    target.append("\\(");
-                    break;
-                case ')':
-                    target.append("\\)");
-                    break;
-                default:
+        switch (c) {
+            case '\n':
+                target.append("\\n");
+                break;
+            case '\r':
+                target.append("\\r");
+                break;
+            case '\t':
+                target.append("\\t");
+                break;
+            case '\b':
+                target.append("\\b");
+                break;
+            case '\f':
+                target.append("\\f");
+                break;
+            case '\\':
+                target.append("\\\\");
+                break;
+            case '(':
+                target.append("\\(");
+                break;
+            case ')':
+                target.append("\\)");
+                break;
+            default:
+                if (c > 255) {
+                    //Ignoring non Latin-1 characters
+                    target.append('?');
+                } else if (c < 32 || c > 127) {
+                    target.append('\\');
+                   
+                    target.append((char)('0' + (c >> 6)));
+                    target.append((char)('0' + ((c >> 3) % 8)));
+                    target.append((char)('0' + (c % 8)));
+                    //Integer.toOctalString(i)
+                } else {
                     target.append(c);
-            }
+                }
         }
     }
 
@@ -238,6 +261,14 @@ public class PSGenerator {
         return convertStringToDSC(text, false);
     }
 
+    /**
+     * Converts a <real> value for use in DSC comments.
+     * @param value the value to convert
+     * @return String The resulting String
+     */
+    public static final String convertRealToDSC(float value) {
+        return Float.toString(value);
+    }
 
     /**
      * Converts text by applying escaping rules established in the DSC specs.
@@ -253,10 +284,7 @@ public class PSGenerator {
             int initialSize = text.length();
             initialSize += initialSize / 2;
             StringBuffer sb = new StringBuffer(initialSize);
-            if ((Long.getLong(text) != null) 
-                    || (text.indexOf(' ') >= 0) 
-                    || forceParentheses) {
-                        
+            if ((text.indexOf(' ') >= 0) || forceParentheses) {
                 sb.append('(');
                 for (int i = 0; i < text.length(); i++) {
                     final char c = text.charAt(i);
@@ -265,7 +293,11 @@ public class PSGenerator {
                 sb.append(')');
                 return sb.toString();
             } else {
-                return text;
+                for (int i = 0; i < text.length(); i++) {
+                    final char c = text.charAt(i);
+                    escapeChar(c, sb);
+                }
+                return sb.toString();
             }
         }
     }
@@ -319,8 +351,8 @@ public class PSGenerator {
                 
                 if (params[i] instanceof String) {
                     tempBuffer.append(convertStringToDSC((String)params[i]));
-                } else if (params[i] instanceof AtendIndicator) {
-                    tempBuffer.append("(atend)");
+                } else if (params[i] == DSCConstants.ATEND) {
+                    tempBuffer.append(DSCConstants.ATEND);
                 } else if (params[i] instanceof Double) {
                     tempBuffer.append(df3.format(params[i]));
                 } else if (params[i] instanceof Number) {
@@ -474,25 +506,71 @@ public class PSGenerator {
             writeln(pattern + " setdash");
         }
     }
-                                
+
     /**
      * Establishes the specified color (RGB).
      * @param col the color as defined by the setrgbcolor command.
      * @exception IOException In case of an I/O problem
+     * @deprecated use useColor method instead
      */
     public void useRGBColor(Color col) throws IOException {
-        if (col == null) {
-            col = PSState.DEFAULT_RGB_COLOR;
-        }
+        useColor(col);
+    }        
+
+    /**
+     * Establishes the specified color.
+     * @param col the color.
+     * @exception IOException In case of an I/O problem
+     */
+    public void useColor(Color col) throws IOException {
         if (getCurrentState().useColor(col)) {
-            float[] comps = col.getColorComponents(null);
-            writeln(formatDouble(comps[0])
-                    + " " + formatDouble(comps[1])
-                    + " " + formatDouble(comps[2])
-                    + " setrgbcolor");
+            writeln(convertColorToPS(col));
         }
     }
     
+    private String convertColorToPS(Color col) {
+        StringBuffer p = new StringBuffer();
+        float[] comps = col.getColorComponents(null);
+        
+        if (col.getColorSpace().getType() == ColorSpace.TYPE_RGB) {
+            // according to pdfspec 12.1 p.399
+            // if the colors are the same then just use the g or G operator
+            boolean same = (comps[0] == comps[1] 
+                        && comps[0] == comps[2]);
+            // output RGB
+            if (same) {
+                p.append(formatDouble(comps[0]));
+            } else {
+                for (int i = 0; i < col.getColorSpace().getNumComponents(); i++) {
+                    if (i > 0) {
+                        p.append(" ");
+                    }
+                    p.append(formatDouble(comps[i]));
+                }
+            }
+            if (same) {
+                p.append(" setgray");
+            } else {
+                p.append(" setrgbcolor");
+            }
+        } else if (col.getColorSpace().getType() == ColorSpace.TYPE_CMYK) {
+            // colorspace is CMYK
+            for (int i = 0; i < col.getColorSpace().getNumComponents(); i++) {
+                if (i > 0) {
+                    p.append(" ");
+                }
+                p.append(formatDouble(comps[i]));
+            }
+            p.append(" setcmykcolor");
+        } else {
+            // means we're in DeviceGray or Unknown.
+            // assume we're in DeviceGray, because otherwise we're screwed.
+            p.append(formatDouble(comps[0]));
+            p.append(" setgray");
+        }
+        return p.toString();
+    }
+    
     /**
      * Establishes the specified font and size.
      * @param name name of the font for the "F" command (see FOP Std Proc Set)
@@ -505,96 +583,63 @@ public class PSGenerator {
         }
     }
     
-    private Set documentSuppliedResources;
-    private Set documentNeededResources;
-    private Set pageResources;
+    private ResourceTracker resTracker = new ResourceTracker();
     
     /**
-     * Notifies the generator that a new page has been started and that the page resource
+     * Resturns the ResourceTracker instance associated with this PSGenerator.
+     * @return the ResourceTracker instance or null if none is assigned
+     */
+    public ResourceTracker getResourceTracker() {
+        return this.resTracker;
+    }
+    
+    /**
+     * Sets the ResourceTracker instance to be associated with this PSGenerator.
+     * @param resTracker the ResourceTracker instance
+     */
+    public void setResourceTracker(ResourceTracker resTracker) {
+        this.resTracker = resTracker;
+    }
+    
+    /**      
+     * Notifies the generator that a new page has been started and that the page resource    
      * set can be cleared.
-     */
+     * @deprecated Use the notifyStartNewPage() on ResourceTracker instead.
+     */      
     public void notifyStartNewPage() {
-        if (pageResources != null) {
-            pageResources.clear();
-        }
+        getResourceTracker().notifyStartNewPage();
+    }
+
+    /**      
+     * Notifies the generator about the usage of a resource on the current page.     
+     * @param res the resource being used    
+     * @param needed true if this is a needed resource, false for a supplied resource    
+     * @deprecated Use the notifyResourceUsageOnPage() on ResourceTracker instead
+     */      
+    public void notifyResourceUsage(PSResource res, boolean needed) {
+        getResourceTracker().notifyResourceUsageOnPage(res);
     }
     
-    /**
-     * Notifies the generator about the usage of a resource on the current page.
-     * @param res the resource being used
-     * @param needed true if this is a needed resource, false for a supplied resource
-     */
-    public void notifyResourceUsage(PSResource res, boolean needed) {
-        if (pageResources == null) {
-            pageResources = new java.util.HashSet();
-        }
-        pageResources.add(res);
-        if (needed) {
-            if (documentNeededResources == null) {
-                documentNeededResources = new java.util.HashSet();
-            }
-            documentNeededResources.add(res);
-        } else {
-            if (documentSuppliedResources == null) {
-                documentSuppliedResources = new java.util.HashSet();
-            }
-            documentSuppliedResources.add(res);
-        }
-    }
-
-    /**
-     * Indicates whether a particular resource is supplied, rather than needed.
-     * @param res the resource
-     * @return true if the resource is registered as being supplied.
-     */
-    public boolean isResourceSupplied(PSResource res) {
-        return documentSuppliedResources.contains(res);
-    }
-
     /**
      * Writes a DSC comment for the accumulated used resources, either at page level or
      * at document level.
      * @param pageLevel true if the DSC comment for the page level should be generated, 
      *                  false for the document level (in the trailer)
      * @exception IOException In case of an I/O problem
+     * @deprecated Use the writeResources() on ResourceTracker instead.
      */
     public void writeResources(boolean pageLevel) throws IOException {
-        if (pageLevel) {
-            writeResourceComment(DSCConstants.PAGE_RESOURCES, pageResources);
-        } else {
-            writeResourceComment(DSCConstants.DOCUMENT_NEEDED_RESOURCES, 
-                    documentNeededResources);
-            writeResourceComment(DSCConstants.DOCUMENT_SUPPLIED_RESOURCES, 
-                    documentSuppliedResources);
-        }
+        getResourceTracker().writeResources(pageLevel, this);
     }
     
-    private void writeResourceComment(String name, Set resources) throws IOException {
-        if (resources == null || resources.size() == 0) {
-            return;
-        }
-        tempBuffer.setLength(0);
-        tempBuffer.append("%%");
-        tempBuffer.append(name);
-        tempBuffer.append(" ");
-        boolean first = true;
-        Iterator i = resources.iterator();
-        while (i.hasNext()) {
-            if (!first) {
-                writeln(tempBuffer.toString());
-                tempBuffer.setLength(0);
-                tempBuffer.append("%%+ ");
-            }
-            PSResource res = (PSResource)i.next();
-            tempBuffer.append(res.getResourceSpecification());
-            first = false;
-        }
-        writeln(tempBuffer.toString());
+    /**
+     * Indicates whether a particular resource is supplied, rather than needed.
+     * @param res the resource
+     * @return true if the resource is registered as being supplied.
+     * @deprecated Use the isResourceSupplied() on ResourceTracker instead.
+     */
+    public boolean isResourceSupplied(PSResource res) {
+        return getResourceTracker().isResourceSupplied(res);
     }
     
-    /** Used for the ATEND constant. See there. */
-    private static interface AtendIndicator {
-    }
-
-
-}
+}
\ No newline at end of file
